/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#include "depend.h"
#include "param.h"
#include "gen.h"

/*============================================================================
 Set appropriate permittion to specified logfile
 [INPUT]
  uid           : User ID of logfile
  filename      : Filename of logfile
 [RETURN]
  PERM_NOERROR  : No Error
  PERM_NOTFOUND : File not found
  PERM_CHOWN    : chown() error
  PERM_CHMOD    : chmod() error 
  ============================================================================
*/
int setperm_logfile(uid_t uid,char *filename)
{
    struct stat     statbuf;

    if (!stat(filename,&statbuf)){
        if (statbuf.st_uid != 0){
            if (chown(filename,0,statbuf.st_gid))
                return(PERM_CHOWN);
        }
        if (statbuf.st_mode != (S_IRUSR|S_IWUSR)){
            if (chmod(filename,S_IRUSR|S_IWUSR))
                return(PERM_CHMOD);
        }
        return(PERM_NOERROR);
    }else
        return(PERM_NOTFOUND);
}

/*============================================================================
 Check logfile permission
 [INPUT]
  fname         : Filename
 [RETURN]
  0             : OK
  <0            : Error
  ============================================================================
*/
int iswriteablefile(char *fname)
{
    FILE            *fp;
    struct stat     statbuf;
    int             r,apflag;
    pid_t           pid;
    unsigned int    md;

    r=stat(fname,&statbuf);
    if ((fp=fopen(fname,"rb"))!=NULL){
        fclose(fp);
        if (r!=0) return(-1);
        md=S_IFMT&(S_IFREG^0xffffffff);
        if (statbuf.st_mode & md) return(-2);
        if (statbuf.st_mode & (S_IRWXO|S_IRWXG)) return(-3);
        if (statbuf.st_mode & S_IXUSR) return(-4);
        if (statbuf.st_uid != 0) return(-5);
        apflag=0;
        if ((pid=fork())==-1){
            fprintf(stderr,"fork() error\n");
            return(-7);
        }else if (pid==0){
            setuid(1);
            if ((fp=fopen(fname,"ab"))!=NULL){
                fclose(fp);
                apflag=1;
            }
            exit(1);
        }else wait(NULL);
        if (apflag) return(-9);
    }else{
        if ((fp=fopen(fname,"wb"))==NULL) return(-6);
        fclose(fp); 
        remove(fname);
        if ((pid=fork())==-1){
            fprintf(stderr,"fork() error\n");
            return(-7);
        }else if (pid==0){
            setuid(1);
            if ((fp=fopen(fname,"wb"))!=NULL){
                fclose(fp); 
            }
            exit(1);
        }else wait(NULL);
        if (!stat(fname,&statbuf)){
            if (statbuf.st_uid !=0 ){
                remove(fname);
                return(-8);
            }
        }
    }
    return(0);
}

/*============================================================================
 Check directory permission
 [INPUT]
  fname         : Filename
 [RETURN]
  0             : OK
  <0            : Error
  ============================================================================
*/
int checkdir(char *dirname)
{
    struct stat     statbuf;
    char            buf[512];
    int             r;
    unsigned int    md;

    if (!mkdir(dirname,S_IXUSR)){
        chown(dirname,0,0);
        chmod(dirname,S_IXUSR);
    }else{
        r=stat(dirname,&statbuf);
        if (r<0) return(-101);
        if (statbuf.st_mode & (S_IRWXO|S_IRWXG)) return(-101);
        if (statbuf.st_uid != 0) return(-102);
        md=S_IFMT&(S_IFDIR^0xffffffff);
        if (statbuf.st_mode & md) return(-103);
    }
    if (strlen(dirname)+2 >= sizeof(buf)) return(-100);
    sprintf(buf,"%s/a",dirname);
    dirname[sizeof(buf)-1]='\0';
    if ((r=iswriteablefile(buf))) return(r);
    return(0);
}
