/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks! : hsj <hsj@shadowpengin.org> helps me to develop this code!
    
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include "depend.h"
#include "param.h"
#include "promisc.h"

#define MAXBUFSIZE	60000

/*============================================================================
 Set network interface to promiscus mode.
 [INPUT]
  nit_dev       : NIT device name
  nic_name      : Name of network interface  
  csize         : Chunk size
 [RETURN]
  ERR_PROMISC_OPEN              : Can not open device 
  ERR_PROMISC_WRITE             : Attach/Promiscus error
  ERR_PROMISC_BIND              : Can not bind (IRIX only)
  ERR_PROMISC_SETSOCKOPT        : setsockopt failure (IRIX only) 
  ERR_PROMISC_TOOLONG_NICNAME   : Too long nic_name
  otherwise                     : device handle
  ============================================================================
*/
SOCKET setnic_promisc(char *nit_dev,char *nic_name,int *csize)
{
        SOCKET          sock;
        int             size=60000,on=1;
        struct          sockaddr_raw sr;
        struct          snoopfilter sf;

        if (strlen(nic_name)>=sizeof(sr.sr_ifname))
		        return(ERR_PROMISC_TOOLONG_NICNAME);

        /* create a snoop socket */
        if((sock=socket(PF_RAW,SOCK_RAW,RAWPROTO_SNOOP))<0)
                return(ERR_PROMISC_OPEN);

        /* bind created socket to desired interface */
        memset(&sr,0,sizeof(sr));
        sr.sr_family = AF_RAW;
        sr.sr_port = 0;
        strcpy(sr.sr_ifname,nic_name);
        if(bind(sock,&sr,sizeof(sr))<0){
                close(sock);
                return(ERR_PROMISC_BIND);
        }

        /* init a filter with no mask bits set, in order to match all packets */
        memset((char *)&sf,0,sizeof(sf));
        if(ioctl(sock,SIOCADDSNOOP,&sf)<0){
                close(sock);
                return(ERR_PROMISC_WRITE);
        }

        /* increase socket's recvbuf size to a generous upper bound */
        if (setsockopt(sock,SOL_SOCKET,SO_RCVBUF,
	    (char *)&size,sizeof(size)))
		return(ERR_PROMISC_SETSOCKOPT);
        if(ioctl(sock,SIOCSNOOPING,&on)<0){
                close(sock);
                return(ERR_PROMISC_WRITE);
        }
        return(sock);
}
